"""
This file is part of LionsTooth.

LionsTooth is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

LionsTooth is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LionsTooth.  If not, see <http://www.gnu.org/licenses/>.
"""

import markdown

from markdown.inlinepatterns import Pattern
from markdown.extensions import Extension
from markdown.util import etree
from markdown.blockparser import BlockParser
from markdown.blockprocessors import BlockProcessor, HashHeaderProcessor
from markdown.treeprocessors import Treeprocessor
from markdown import util
import re

"""
https://python-markdown.github.io/extensions/api/
"""
class mention_pattern(Pattern):
	def __init__(self, pattern):
		Pattern.__init__(self, pattern)
			
	def handleMatch(self, m):
		b = etree.Element('b')
		b.text = "@"
		a = etree.SubElement(b, 'a') 
		a.text = markdown.util.AtomicString(m.group(2)) # AtomicString doesn't parse text
		a.set("href", "diaspora_id://{}".format(markdown.util.AtomicString(m.group(3))))
		return b
		
class tag_pattern(Pattern):
	def __init__(self, pattern):
		Pattern.__init__(self, pattern)
			
	def handleMatch(self, m):
		b = etree.Element('b')
		a = etree.SubElement(b, 'a') 
		a.text = markdown.util.AtomicString("#{}".format(m.group(2))) # AtomicString doesn't parse text
		a.set("href", "tag://{}".format(markdown.util.AtomicString(m.group(2))))
		return b
	
class diaspora_md(Extension):
	def __init__(self, **kwargs):
		Extension.__init__(self, **kwargs)
		
		self.pattern = {}
		self.pattern.update({ 'author_name'	: r'.+?[^;]' } )# TODO
		self.pattern.update({ 'diaspora_id'	: r'[a-zA-Z0-9_\-\.]+@[a-zA-Z0-9_\-\.]+\.[a-zA-Z]{2,10}' } )# TODO
		self.pattern.update({ 'mention'		: r'\@\{(%(author_name)s)\s?\;\s?(%(diaspora_id)s)\}' % {'author_name':self.pattern['author_name'], 'diaspora_id':self.pattern['diaspora_id'] }})
		self.pattern.update({ 'tag'			: r'#([\w0-9\_\-]{1,255})' } )
	
	def extendMarkdown(self, md, md_globals):
		obj_mention_pattern = mention_pattern(self.pattern['mention'])
		obj_tag_pattern = tag_pattern(self.pattern['tag'])
		
		md.inlinePatterns.add('mention', obj_mention_pattern, '<reference')
		md.inlinePatterns.add('hashtag', obj_tag_pattern, '<reference')
		md.parser.blockprocessors['hashheader'].RE = re.compile(r'(^|\n)(?P<level>#{1,6} )(?P<header>.*?)#*(\n|$)') # hashheader now requires a space between the # and the text
