"""
This file is part of LionsTooth.

LionsTooth is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

LionsTooth is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LionsTooth.  If not, see <http://www.gnu.org/licenses/>.
"""

import re
from collections import OrderedDict
from classes.workarounds import workarounds

#DEBUG
import time

class markdown_obj_parser():
	def __init__(self):
		self.pattern = OrderedDict()

		self.workaround = workarounds()

		""" Objects patterns """
		url_patterns = {}
		url_patterns.update( { 'scheme'	: r'https?://' } )
		url_patterns.update( { 'netloc'	: r'([a-zA-Z0-9_\-]+\.)+[a-zA-Z]{2,13}' } )
		url_patterns.update( { 'params'	: r'[a-zA-Z0-9\.\/_\-\:\%\=\&\?\,\@\#\$\!\^\*\(\)\+]+' } )
		url_patterns.update( { 'url'	: r'({})({})/({})'.format(url_patterns['scheme'], url_patterns['netloc'], url_patterns['params']) } )

		self.pattern = {}
		self.pattern.update( { 'image'		: r'\!\[(.*?)\]\(({0})( [\"\'](.+)[\"\'])?\)'.format(url_patterns['url']) } )
		self.pattern.update( { 'image_link'	: r'\[{0}\]\((https?://[a-zA-Z0-9\.\/_\-\:\%\=\&\?\,\@\#\$\!\^\*\(\)\+]+)( [\"\'](.+)[\"\'])?\)'.format(self.pattern['image']) } )
		self.pattern.update( { 'image'		: r'(?<!\[){}'.format(self.pattern['image']) } )
		self.pattern.update( { 'code_block'	: r'[`]{3,4}(.*?)[`]{3,4}' } )

	def parse(self, string):
		string = self.replace_references(string)
		matches = [] #[[start,end,match,pattern_name], ..]
		for name in self.pattern:
			if name == 'code_block': p = re.compile(self.pattern[name],re.I|re.DOTALL|re.M)
			else: p = re.compile(self.pattern[name],re.I)
			for m in p.finditer(string):
				new_obj = self.workaround.groups_to_substitution( m )
				matches.append([m.start(), len(m.group())+m.start(), new_obj, name])

		objects = []
		last_match_end = 0
		for data in sorted(matches):
			text = string[last_match_end:data[0]]
			if len(text.strip()): objects.append(['text',text])

			if data[3] == 'image': objects.append([data[3], data[2][2], data[2][1], data[2][7]]) # obj_name, url, alt_text, title
			elif data[3] == 'image_link': objects.append([data[3], data[2][2], data[2][7], data[2][8], data[2][1], data[2][9], data[2][10]])
			elif data[3] == 'code_block' and data[2][1].strip(): objects.append([data[3], data[2][1]])

			obj_str = string[data[0]:data[1]]
			last_match_end = data[1]

		text = string[last_match_end:]
		if len(text.strip()): objects.append(['text',text])
		return objects

	def replace_references(self, text):
		references = re.findall(r'(\[([a-zA-Z0-9]+)\]\: (.*?)[\s\n\r]+)', text, re.M)
		if references:
			for ref in references:
				ref_groups = self.workaround.groups_to_substitution( ref )
				text = text.replace(ref_groups[0], '')
				text = text.replace('[{}]'.format(ref_groups[1]), '({})'.format(ref_groups[2]))
		return text
