"""
This file is part of LionsTooth.

LionsTooth is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

LionsTooth is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LionsTooth.  If not, see <http://www.gnu.org/licenses/>.
"""

""" For fetching images """
import certifi # https
import urllib3
from urllib.parse import urlparse
from os import path, makedirs
import hashlib

from requests import ConnectionError
from requests.exceptions import SSLError

from PyQt5.QtCore import QObject, QRunnable, QThreadPool, pyqtSignal

""" people worker """
class img_signals(QObject):
	finished = pyqtSignal(str)
	finished_index = pyqtSignal([int, str])
	exception = pyqtSignal(str)

# TODO exception handling
class img_worker(QRunnable):
	def __init__(self, res=None, html=None, url=None, index=None):
		super(img_worker, self).__init__()
		self.res = res
		self.url = url
		self.html = html
		self.index = index
		self.signals = img_signals()
		
	def get_image(self, url):
		parsed_url = urlparse(url)
		hash_object = hashlib.md5(parsed_url.path.encode('utf-8'))
		# TODO look into to replace the md5 with xxhash https://cyan4973.github.io/xxHash/
		
		filepath = '{}/{}/{}'.format(self.res.img_path, parsed_url.netloc, hash_object.hexdigest())
		if path.exists(filepath):
			#print("Img from cache \t{}".format(filepath)) # DEBUG
			return filepath

		# Create image url
		img_url = ""
		if not parsed_url.scheme:
			img_url += "https://"
		else:
			img_url += parsed_url.scheme + "://"
			
		if not parsed_url.netloc:
			pod = urlparse(self.res.netloc)
			img_url += pod.netloc + "/"
		else:
			img_url += parsed_url.netloc
			
		if not parsed_url.path:
			print("resolver.py says this is not good")
		else:
			img_url += parsed_url.path

		# camo doesn't support HEAD
		if not self.res.camo_enabled:
			# Check content-type and content-size
			max_size = 10485760 # 10MB, TODO add option in settings to change this value.
			try:
				head_response = self.res.http.request('HEAD', img_url)
			except (ConnectionError, SSLError) as e:
				print("1 Could not get img headers: {} {}".format(img_url, e))
				return ""
			except:
				print("2 Could not get img headers: {}".format(img_url))
				return ""

			if head_response.headers['Content-Type'].lower() not in ['image/png','image/jpeg','image/gif']:
				print("[ Unsupported image type ]")
				print(img_url) # DEBUG
				print(head_response.headers) # DEBUG
				print() # DEBUG
				return ""

			if int(head_response.headers['Content-Length']) > max_size:
				print("Image to big")
				print(head_response.headers) # DEBUG
				return ""

		# Get image
		# TODO verify that what we are downloading is a image, and set max size
		try:
			response = self.res.http.request('GET', img_url)
		except (ConnectionError, SSLError) as e:
			print("1 Could not get img: {} {}".format(img_url, e))
			return ""
		except:
			print("2 Could not get img: {}".format(img_url))
			return ""
			
		img_data = response.data
		
		# Check if dir path exists
		dirpath = '/'.join(filepath.split('/')[0:-1])
		if not path.isdir(dirpath): # If dir doesn't exist, create it.
			makedirs(dirpath)
		
		# Write image
		img_file = open(filepath, "wb")
		img_file.write(img_data)
		img_file.close()
		
		# Return path to image.
		#print("Remote Img cached \t{}".format(filepath)) # DEBUG
		return filepath
	
	def get_img_from_html(self):
		import re
		img_tags = re.findall("((<img.*?)(src=\"(.*?)\")(.*?\/>))", self.html, re.I)
		for img_tag in img_tags:
			path = self.get_image( img_tag[3] )
			if path:
				new_img_tag = img_tag[0].replace(img_tag[3], path)
				self.html = self.html.replace(img_tag[0], new_img_tag)
		
	def run(self):
		if self.url != None:
			try:
				path = self.get_image(self.url)
			except:
				self.signals.exception.emit("Resolver: could not get img")
				#print("exception Resolver: could not get img") # DEBUG
			else:
				if self.index != None:
					self.signals.finished_index.emit(self.index, path)
				else:
					self.signals.finished.emit(path)
		elif self.html != None:
			try:
				self.get_img_from_html()
			except:
				self.signals.exception.emit("Resolver: could not get images")
			else:
				self.signals.finished.emit(self.html)

class resolver():
	def __init__(self, netloc, img_path='./cache/'):
		self.http = urllib3.PoolManager(cert_reqs='CERT_REQUIRED', ca_certs=certifi.where())
		self.netloc = netloc
		self.img_path = img_path
		self.camo_enabled = False
		self.thread_pool = QThreadPool()
		self.thread_pool.setMaxThreadCount(264)

		print("{0}/camo/".format(netloc))
		try:
		
			response = self.http.request('GET', "{0}/camo/".format(netloc))
			if response.status == 200:
				self.camo_enabled = True
				print("Camo enabled. Not requesting HEAD before CONTENT.") # DEBUG MSG
			else: print("Camo disabled. Do request HEAD before CONTENT") # DEBUG MSG
			# TODO option to set proxy for non https content if camo disabled.
		except (ConnectionError, SSLError) as err:
			print("Connection error, could not check on camo: {0}".format(err))
