# -*- coding: utf-8 -*-
"""
This file is part of LionsTooth.

LionsTooth is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

LionsTooth is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LionsTooth.  If not, see <http://www.gnu.org/licenses/>.
"""
from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtCore import pyqtSignal

from widgets.stream import stream as stream_widget

class header(QtWidgets.QFrame):
	request_follow_tag = pyqtSignal(str) # str: tag name
	request_unfollow_tag = pyqtSignal(int, str) # int: id, str: tag name
	def __init__(self, parent=None):
		QtWidgets.QFrame.__init__(self, parent=parent)

		self.tag_name = "#"
		self.tag_id = False

		h_layout = QtWidgets.QHBoxLayout(self)
		self.setLayout( h_layout )

		self.tag_label = QtWidgets.QLabel(self)
		
		self.follow_tag_btn = QtWidgets.QPushButton(self)
		sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
		self.follow_tag_btn.setSizePolicy(sizePolicy)
		
		self.layout().addWidget(self.tag_label)
		self.layout().addWidget(self.follow_tag_btn)
		
		self.follow_tag_btn.clicked.connect( self.toggle_follow )

		self.setMaximumHeight(60)
		
	def toggle_follow(self):
		if self.tag_id != None:
			self.unfollow_tag()
		else:
			self.follow_tag()

	def follow_tag(self):
		self.request_follow_tag.emit( self.tag_name )

	def tag_followed(self, tag_id):
		self.tag_id = tag_id
		self.follow_tag_btn.setStyleSheet("background-color: #b1ff58;\n"
		"color: #000;")
		self.follow_tag_btn.setText( "Following #{}".format( self.tag_name ) )
		self.follow_tag_btn.setToolTip("Click to unfollow #{}".format( self.tag_name ))
	
	def unfollow_tag(self):
		self.request_unfollow_tag.emit( self.tag_id, self.tag_name )
		
	def tag_unfollowed(self):
		self.tag_id = None
		self.follow_tag_btn.setStyleSheet("background-color: #2c2c2c;\n"
		"color: #fff;")
		self.follow_tag_btn.setText( "Follow #{}".format( self.tag_name ) )
		self.follow_tag_btn.setToolTip("Not following #{}.\nClick to follow #{}".format( self.tag_name, self.tag_name ))
		
	def set_tag_name(self, name, following_tag=None):
		self.tag_name = "{}".format( name )
		self.tag_label.setText( "<h2>#{}</h2>".format( self.tag_name ) )
		if following_tag != None:
			self.tag_id = following_tag
			self.tag_followed(self.tag_id)
		else:
			self.tag_unfollowed()

class followed_tags(stream_widget):
	def __init__(self, thread_pool, parent=None):
		stream_widget.__init__(self, parent=parent)
		self.name = "followed_tags"
		self.thread_pool = thread_pool

		self.current_tag = None
		self.is_busy = False

		self.header = header(self)
		self.ui.stream_layout.insertWidget(0, self.header )

		self.post_offset = 1 # first widget will contain tag name and a button if not in followed_tags

	def set_not_busy(self):
		self.is_busy = False

	def set_tag(self, name, following_tag):
		self.current_tag = name
		self.header.set_tag_name( name, following_tag=following_tag )
