"""
This file is part of LionsTooth.

LionsTooth is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

LionsTooth is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LionsTooth.  If not, see <http://www.gnu.org/licenses/>.
"""
from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtCore import pyqtSignal

from design.ui_people_results import Ui_people_results
from design.ui_contact import Ui_contact

from widgets.shared.author_name import author_name
from classes.resolver import img_worker

class user_result(QtWidgets.QFrame):
	user_clicked = pyqtSignal(str, str)# guid, diaspora_id
	def __init__(self, userdata, res=None, parent=None):
		QtWidgets.QFrame.__init__(self, parent)
		self.ui = Ui_contact()
		self.ui.setupUi(self)

		self._res = res
		self._userdata = userdata
		""" self._userdata:
		{
			"id"	:{ID},
			"guid"	:"{GUID}",
			"name"	:"{NAME}",
			"avatar":"{URL}",
			"handle":"{DIASPORA_ID}",
			"url"	:"/people/{GUID}"
		}
		"""
		# Set stylesheet TODO palette's
		self.setStyleSheet("background-color: rgb(21, 30, 20);")
		
		# Name
		self.ui.name = author_name(text=userdata['name'], guid=userdata['guid'], diaspora_id=userdata['handle'])
		self.ui.content_layout.insertWidget( 0, self.ui.name )

		# Diaspora ID
		self.ui.diaspora_id.setText( userdata['handle'] )

		# Avatar
		self.ui.avatar.setMinimumWidth(50)
		self.ui.avatar.setMaximumWidth(50)

		self.ui.avatar.setMinimumHeight(50)
		self.ui.avatar.setMaximumHeight(50)
		if self._res:
			avatar_worker = img_worker(self._res, url=userdata['avatar'])
			avatar_worker.signals.finished.connect( self.set_avatar )
			self._res.thread_pool.start(avatar_worker)

		# Hooks
		self.ui.name.clicked.connect( self.user_clicked )

	def set_res(self, res):
		if res: self._res = res

	def set_avatar(self, path):
		pixmap = QtGui.QPixmap(path)
		if pixmap:
			self.ui.avatar.setPixmap( pixmap )

class people_results(QtWidgets.QWidget):
	user_clicked = pyqtSignal(str, str)# guid, diaspora_id
	anchor_clicked = pyqtSignal(str) # tag name
	def __init__(self, parent=None):
		QtWidgets.QWidget.__init__(self, parent)
		self.ui = Ui_people_results()
		self.ui.setupUi(self)
		self._res = None

		self.ui.header.setStyleSheet("background-color: rgb(21, 30, 20);")
		self.ui.header.linkActivated.connect( self.anchor_clicked )

	def set_res(self, res):
		if res: self._res = res

	def clear_layout(self):
		for i in range(0,self.ui.results_layout.count()):
			item = self.ui.results_layout.itemAt(i)
			if not item: continue
			w = item.widget()
			if w and type(w) is user_result:
				w.user_clicked.disconnect(self.user_clicked)
				w.deleteLater()

	def set_header(self, query):
		if query:
			link_css = "style=\"color: white;\""
			self.ui.header.setText("<h3>Search results for {0}</h3> <b>Looking for messages with <a href='tag://{0}' {1}>#{0}</a>?</b>".format(query, link_css))
		else:
			self.ui.header.hide()

	def set_results(self, results, query=None):
		self.clear_layout() # clear previous
		self.set_header(query)
		for user in results:
			new_user = user_result(user, res=self._res)
			new_user.user_clicked.connect(self.user_clicked)
			self.ui.results_layout.addWidget(new_user)
