# -*- coding: utf-8 -*-
"""
This file is part of LionsTooth.

LionsTooth is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

LionsTooth is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LionsTooth.  If not, see <http://www.gnu.org/licenses/>.
"""
from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtCore import pyqtSignal, pyqtSlot

from widgets.stream import stream as stream_widget
from widgets.markdown_editor import markdown_editor

from classes.handler import stream_worker

from diaspy.models import Post as diaspy_post
import json

class stream(stream_widget):
	update_request = pyqtSignal(str)# stream name
	def __init__(self, thread_pool, parent=None):
		stream_widget.__init__(self, parent=parent)
		self.name = "stream"
		self.thread_pool = thread_pool

		self.markdown_editor = markdown_editor()
		self.ui.stream_layout.insertWidget(0, self.markdown_editor)
		
		self.post_offset = 1 # for the editor
		
		self.markdown_editor.submit_request.connect( self.post )
		
		self.own_avatar_updated.connect( self.markdown_editor.set_avatar )
		
		self.res_set.connect( self.set_markdown_editor_res )
		self.search_set.connect( self.set_markdown_editor_search )

	def set_markdown_editor_search(self):
		self.search_set.disconnect()
		self.markdown_editor.set_search( self.search )

	def set_markdown_editor_res(self): # one time function
		self.res_set.disconnect()
		self.markdown_editor.set_res( self.res )

	def set_max_width(self, max_width):
		self.max_width = max_width
		self.markdown_editor.set_max_width( max_width )

	def post(self, ids, markdown, photo='', photos=None, poll_question=None, poll_answers=None):
		""" Publish a post

		:param poll_question: Question string
        :type poll_question: str

        :param poll_answers: Anwsers to the poll
        :type poll_answers: list with strings
		"""
		if self.stream:
			post_data = {
				'text'			: markdown, 
				'aspect_ids'	: ids, 
				'photos'		: photos, 
				'photo'			: photo,
				'poll_question'	: poll_question,
				'poll_answers'	: poll_answers
			}
			worker = stream_worker('post', stream=self.stream, post_data=post_data)
			worker.signals.new_post.connect( self.new_post )
			worker.signals.exception.connect( self.markdown_editor.not_published )
			#worker.signals.exception.connect( self.msg.put oid.. )
			
			self.thread_pool.start(worker)

	def new_post(self, post):
		if post:
			self.markdown_editor.published()
			self.stream._stream.insert(0, post)
			self.insert_post_widgets()
