# -*- coding: utf-8 -*-
"""
This file is part of LionsTooth.

LionsTooth is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

LionsTooth is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LionsTooth.  If not, see <http://www.gnu.org/licenses/>.
"""
from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtCore import pyqtSignal

from design.ui_header import Ui_header
from widgets.shared.author_name import author_name
from classes.resolver import img_worker

class update_icon(QtWidgets.QLabel):
	clicked = pyqtSignal()
	def __init__(self, parent=None):
		QtWidgets.QLabel.__init__(self, parent)
		
		self.resize(24,24)
		
		self.setObjectName("update_icon")
		self.setFrameShape(QtWidgets.QFrame.StyledPanel)
		
		self.loading_gif = QtGui.QMovie(":/icons/icons/loading.gif")
		self.loading_gif.setScaledSize(QtCore.QSize(24, 24))
		self.setMovie(self.loading_gif)
		self.loading_gif.start()
		self.loading_gif.setPaused(True)

	def start(self):
		self.loading_gif.start()

	def pause(self):
		self.loading_gif.setPaused(True)

	def mousePressEvent(self, event):
		self.clicked.emit()

class user_model(QtGui.QStandardItemModel):
	def __init__(self):
		QtGui.QStandardItemModel.__init__(self)
		self._res = None
		self._max_text_width = 0
		self._font_metrics = 0 # font metrics
		self.calc_font_metrics()
		self.avatar_workers = []

	def calc_font_metrics(self):
		font_test = QtGui.QStandardItem("Test").font()
		self._font_metrics = QtGui.QFontMetrics(font_test)

	def calc_text_width(self, text):
		text_size = self._font_metrics.size(0, text)
		return text_size.width() 

	def set_usersdata(self, usersdata):
		self.setRowCount(len(usersdata))
		self.setColumnCount(1)# avatar: name\nhandle
		self._max_text_width = 0
		index = 0
		for userdata in usersdata:
			main_item = QtGui.QStandardItem( "{}\n{}".format(userdata['name'], userdata['handle']) )
			main_item.setIcon(QtGui.QIcon(userdata['avatar']))
			main_item.setRowCount(2)

			name_width = self.calc_text_width(userdata['name'])
			handle_width = self.calc_text_width(userdata['handle'])
			if name_width > self._max_text_width: self._max_text_width = name_width
			if handle_width > self._max_text_width: self._max_text_width = handle_width
			
			self.setItem(index, 0, main_item)

			# avatar thread
			if self._res:
				avatar_worker = img_worker(self._res, index=index, url=userdata['avatar'])
				avatar_worker.signals.finished_index.connect( self.set_avatar )
				avatar_worker.signals.finished_index.connect( self.finished_avatar )
				self.avatar_workers.append( avatar_worker )
				self._res.thread_pool.start(avatar_worker)
			
			index +=1

	def finished_avatar(self, index, path):
		self.avatar_workers[index] = None

	def set_avatar(self, index, path):
		if self.item(index): self.item(index).setIcon(QtGui.QIcon(path))

	def set_res(self, res): self._res = res

class search_completer(QtWidgets.QCompleter):
	def __init__(self, parent=None):
		QtWidgets.QCompleter.__init__(self, parent=parent)
		self.setCaseSensitivity(0)
		self.setCompletionMode(1) # QCompleter::UnfilteredPopupCompletion

		self.user_model = user_model()
		self.tag_model = QtCore.QStringListModel()

		sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
		self.popup().setSizePolicy(sizePolicy)
		self.popup().setMaximumWidth(16000)

	def clear_models(self):
		self.user_model.clear()
		self.tag_model.setStringList([])

	def set_res(self, res):
		self.user_model.set_res(res)

	def pathFromIndex(self, index):
		""" since we set {name}\n{diaspora_id} and only want the
		{name} for user_model(). for tag model (QStringListModel) just
		return text """
		data = self.model().itemData(index)
		text = data[0]
		if type(self.model()) is user_model: return text.split('\n')[0]
		return text

class header(QtWidgets.QWidget):
	request_people_results_page = pyqtSignal(object, str) # result, query
	request_tag_results_page = pyqtSignal(object, str) # result, query
	def __init__(self, parent=None):
		QtWidgets.QWidget.__init__(self, parent=parent)
		self.ui = Ui_header()
		self.ui.setupUi(self)

		self.max_width = None
		self.userdata = None
		self._search = None

		self.icon = {}
		self.icon['notification_normal'] = QtGui.QIcon()
		self.icon['notification_active'] = QtGui.QIcon()
		self.icon['notification_normal'].addPixmap(QtGui.QPixmap(":/icons/icons/notify_white.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
		self.icon['notification_active'].addPixmap(QtGui.QPixmap(":/icons/icons/notify_red.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)

		self.icon['message_normal'] = QtGui.QIcon()
		self.icon['message_active'] = QtGui.QIcon()
		self.icon['message_normal'].addPixmap(QtGui.QPixmap(":/icons/icons/msg_48_white.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
		self.icon['message_active'].addPixmap(QtGui.QPixmap(":/icons/icons/msg_48_red.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)

		self.author_name = author_name()
		self.author_name.set_text("Disconnected")

		self.author_name.font = QtGui.QFont("FreeSans", 8)
		self.author_name.font.setUnderline(True)
		self.author_name.setFont(self.author_name.font)
		self.ui.header_layout.insertWidget(1, self.author_name)

		self.update_icon = update_icon(self.ui.frame)
		self.ui.header_layout.insertWidget(4, self.update_icon)

		self.notification_count = 999 # so it inits
		self.message_count = 999

		# Search suggestions
		self.search_completer = search_completer(self)
		self.search_completer.setCaseSensitivity(0)
		
		self.ui.le_search.setCompleter( self.search_completer )
		self._last_results = None
		self._last_query = None

	def set_res(self, res): self.search_completer.set_res(res)

	def set_max_width(self, max_width):
		self.max_width = max_width

	def search_text_changed(self, text):
		if self._search:
			if len(text) > 3:
				# if any avatar workers while preparing for new search
				if self.search_completer.user_model.avatar_workers:
					for worker in self.search_completer.user_model.avatar_workers:
						if worker:
							try:
								worker.signals.finished_index.disconnect() # disconnect worker signals
							except TypeError:
								pass
						# TODO cancel thread
				
				result = None
				if text[0:1] == '#': # do tag search
					result = self._search.tags( text[1:], limit=25 ) # TODO thread
					if result:
						self.search_completer.setModel( self.search_completer.tag_model )
						self.search_completer.tag_model.setStringList(result)
						self._last_results = result
						self._last_query = text[1:]

				else: # do people search
					if text[0:1] == '@': # TODO thread
						result = self._search.user( text[1:] )
						self._last_query = text[1:]
					else:
						result = self._search.user( text )
						self._last_query = text

					if result:
						self._last_results = result
						self.search_completer.setModel( self.search_completer.user_model )
						self.search_completer.user_model.set_usersdata( result )
						self.search_completer.popup().setMinimumWidth(self.search_completer.user_model._max_text_width)
						# TODO move popup to left and check if it fits
			else:
				self.search_completer.clear_models() # clear items

	def search(self):
		if self._last_results:
			text = self.ui.le_search.text()
			if text[0:1] == '#': self.request_tag_results_page.emit(self._last_results, self._last_query)
			else: self.request_people_results_page.emit(self._last_results, self._last_query)
			self.ui.le_search.setText("")

	def set_search(self, search):
		self._search = search
		# Hooks
		self.ui.le_search.returnPressed.connect( self.search )
		self.ui.le_search.textChanged.connect( self.search_text_changed )

	def update_avatar(self, path):
		pixmap = QtGui.QPixmap(path)
		if pixmap:
			self.ui.lbl_avatar.setPixmap(pixmap)

	def update_userdata(self, new_userdata):
		self.userdata = new_userdata

	def notification_toggled(self, bool):
		count = self.notification_count
		if bool:
			if count > 0:
				count -=1
		else:
			count +=1
		self.set_notification_count( count )

	def set_notification_count(self, count):
		self.notification_count = int(count)
		if count > 0:
			self.ui.btn_notify.setText( str(count) )
			self.ui.btn_notify.setIcon(self.icon['notification_active'] )
		else:
			self.ui.btn_notify.setText( "" )
			self.ui.btn_notify.setIcon(self.icon['notification_normal'] )
		self.ui.btn_notify.setIconSize(QtCore.QSize(32, 32))
			
	def set_message_count(self, count):
		self.message_count = count
		if count > 0:
			self.ui.btn_msg.setIcon(self.icon['message_active'] )
			self.ui.btn_msg.setText( str(count) )
		else:
			self.ui.btn_msg.setIcon(self.icon['message_normal'] )
			self.ui.btn_msg.setText( "" )
		self.ui.btn_msg.setIconSize(QtCore.QSize(32, 32))
