"""
This file is part of LionsTooth.

LionsTooth is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

LionsTooth is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LionsTooth.  If not, see <http://www.gnu.org/licenses/>.
"""
from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtCore import pyqtSignal

from design.ui_markdown_editor import Ui_markdown_editor

from design.ui_poll_edit import Ui_poll_edit
from design.ui_poll_edit_option import Ui_poll_edit_option

from widgets.markdown_viewer import markdown_viewer, image
from widgets.aspects_button import aspects_button
from widgets.shared.loading_icon import loading_icon

from classes.markdown_obj_parser import markdown_obj_parser
from classes.resolver import img_worker

from classes.markdown_extention import diaspora_md
import markdown
import re

from classes.resolver import img_worker

from widgets.shared.author_name import author_name

class poll_edit_option(QtWidgets.QFrame):
	delete_option = pyqtSignal(int) # self.id
	def __init__(self, id, parent=None):
		QtWidgets.QFrame.__init__(self, parent)
		self.ui = Ui_poll_edit_option()
		self.ui.setupUi(self)

		self.id = id # temporary id
		self.ui.btn_delete_poll_answer.hide()

		self.ui.btn_delete_poll_answer.clicked.connect(self._delete_option)

	def _delete_option(self): self.delete_option.emit(self.id)

class poll_edit(QtWidgets.QFrame):
	def __init__(self, parent=None):
		QtWidgets.QFrame.__init__(self, parent)
		self.ui = Ui_poll_edit()
		self.ui.setupUi(self)

		self.options = {}
		self.option_id_counter = -1
		self._delete_buttons_visible = False

		# Add default options (minimal 2)
		for i in range(0,2): self.add_option()

		# Hooks
		self.ui.btn_add_option.clicked.connect( self.add_option )

	def data(self):
		question_str = self.ui.poll_question.text().strip()
		if question_str:
			answer_list = []
			for id in self.options:
				answer = self.options[id].ui.poll_answer.text().strip()
				if answer: answer_list.append(str(answer))
			if len(answer_list) > 1: return (question_str, answer_list)
		return None

	def toggle_delete_buttons(self):
		if len(self.options) > 2:
			for id in self.options: self.options[id].ui.btn_delete_poll_answer.show()
			self._delete_buttons_visible = True
		elif self._delete_buttons_visible:
			for id in self.options: self.options[id].ui.btn_delete_poll_answer.hide()
			self._delete_buttons_visible = False

	def add_option(self):
		self.option_id_counter +=1
		option = poll_edit_option(self.option_id_counter)
		option.delete_option.connect( self.del_option ) # connect delete button
		self.ui.poll_options.addWidget(option)
		self.options.update({ self.option_id_counter:option })
		self.toggle_delete_buttons()

	def del_option(self,id):
		self.options[id].delete_option.disconnect( self.del_option )
		self.options[id].deleteLater()
		self.options.pop(id)
		self.toggle_delete_buttons()

class deletable_thumb(QtWidgets.QLabel):
	delete_photo = pyqtSignal(int) # photo_id
	def __init__(self, id, path, parent=None):
		QtWidgets.QLabel.__init__(self, parent)
		self.id = id
		self.setMaximumSize(QtCore.QSize(16777215, 150))
		self.setFrameShape(QtWidgets.QFrame.StyledPanel)
		
		sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
		sizePolicy.setHorizontalStretch(0)
		sizePolicy.setVerticalStretch(0)
		sizePolicy.setHeightForWidth(self.sizePolicy().hasHeightForWidth())
		self.setSizePolicy(sizePolicy)

		pixmap = QtGui.QPixmap(path)
		if pixmap: self.setPixmap( pixmap.scaled(150,150,QtCore.Qt.KeepAspectRatio) )

		# Delete button
		self.delete_button = QtWidgets.QPushButton(self)
		icon = QtGui.QIcon()
		icon.addPixmap(QtGui.QPixmap(":/icons/icons/delete_thumb.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
		self.delete_button.setIcon(icon)
		self.delete_button.setFlat(True)
		self.delete_button.setIconSize( QtCore.QSize(48, 48) )
		self.delete_button.setToolTip("Delete this photo.")
		self.delete_button.hide()

		# Delete button hook
		self.delete_button.clicked.connect( self._delete_photo )

	def _delete_photo(self): self.delete_photo.emit( self.id )

	def enterEvent(self, event):
		self.delete_button.show()
		x = self.size().width() / 2
		x -= self.delete_button.size().width() / 2
		y = self.size().height() / 2
		y -= self.delete_button.size().height() / 2
		self.delete_button.move(x,y)
		
	def leaveEvent(self, event): self.delete_button.hide()

class mentions_box(QtWidgets.QFrame):
	user_clicked = pyqtSignal(str) # str: handle
	def __init__(self, mentions=[], parent=None):
		QtWidgets.QFrame.__init__(self, parent=parent)
		self.setWindowTitle("Mention search")
		
		""" gettings avatars """
		self._res = None

		""" layout """
		verticalLayout = QtWidgets.QVBoxLayout(self)
		verticalLayout.setSpacing(0)
		verticalLayout.setContentsMargins(0, 0, 0, 0)

		self.scrollArea = QtWidgets.QScrollArea(self)
		self.scrollArea.setWidgetResizable(True)
		self.scrollAreaWidgetContents = QtWidgets.QWidget()
		self.scrollAreaWidgetContents.setGeometry( QtCore.QRect(0, 0, 502, 313) )

		self.mentions_layout = QtWidgets.QVBoxLayout( self.scrollAreaWidgetContents )
		self.mentions_layout.setSpacing(0)
		self.mentions_layout.setContentsMargins(0, 0, 0, 0)
		self.scrollArea.setWidget( self.scrollAreaWidgetContents )

		verticalLayout.addWidget( self.scrollArea )

		""" mentions """
		self.mentions = mentions # list with dicts

		self.hide()
		self.move(self.pos().x(), self.pos().y()+50)

	def set_res(self, res):
		self._res = res

	def set_mentions(self, mentions):
		if mentions:
			self.clear_layout()
			self.mentions = mentions

			for mention in mentions:
				m_card = mention_card(mention)
				m_card.user_clicked.connect( self.user_clicked )
				
				avatar_worker = img_worker( self._res, url=mention['avatar'] )
				avatar_worker.signals.finished.connect( m_card.update_avatar )
				self._res.thread_pool.start( avatar_worker )
				
				self.mentions_layout.addWidget(m_card)

	def clear_layout(self):
		while self.mentions_layout.count():
			item = self.mentions_layout.takeAt(0)
			if not item: continue
			w = item.widget()
			if w:
				w.user_clicked.disconnect()
				w.deleteLater()

class mention_card(QtWidgets.QFrame):
	user_clicked = pyqtSignal(str) # handle
	def __init__(self, userdata, parent=None):
		QtWidgets.QFrame.__init__(self, parent=parent)

		self.userdata = userdata
		"""
		{
			"id":123,
			"guid":"{GUID}",
			"name":"Test name",
			"avatar":"https://niceUrl.com/8.jpg",
			"handle":"something@niceUrl.net",
			"url":"/people/{GUID}"
		}
		"""

		""" layout """
		horizontalLayout = QtWidgets.QHBoxLayout(self)
		
		# Size policy
		sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
		self.setSizePolicy(sizePolicy)

		# Avatar
		self.avatar = QtWidgets.QLabel(self)
		self.avatar.setMinimumSize(QtCore.QSize(50, 50))
		self.avatar.setMaximumSize(QtCore.QSize(50, 50))

		horizontalLayout.addWidget(self.avatar)

		verticalLayout = QtWidgets.QVBoxLayout()

		# Name
		self.name = author_name(guid=userdata['guid'], diaspora_id=userdata['handle'], text=userdata['name'], parent=self)
		self.name.clicked.connect( self.click ) # when username is clicked
		verticalLayout.addWidget(self.name)

		# Diaspora_id / Handle
		self.diaspora_id = QtWidgets.QLabel(self)
		self.diaspora_id.setText(userdata['handle'])
		verticalLayout.addWidget(self.diaspora_id)

		horizontalLayout.addLayout(verticalLayout)

	def update_avatar(self, path):
		if path:
			pixmap = QtGui.QPixmap(path)
			if pixmap:
				self.avatar.setMaximumSize(QtCore.QSize(50, 50))
				self.avatar.setMinimumSize(QtCore.QSize(50, 50))
				self.avatar.setPixmap(pixmap)

	def click(self):
		self.user_clicked.emit( self.userdata['handle'] )

	def mousePressEvent(self, event):
		self.click()
		QtWidgets.QFrame.mousePressEvent(self, event)

class edit(QtWidgets.QPlainTextEdit):
	clicked = pyqtSignal()
	def __init__(self, parent=None):
		QtWidgets.QPlainTextEdit.__init__(self, parent=parent)
		self.setPlaceholderText("Write a message...")
		
		self._search = None
		self._res = None
		self.last_query = ""
		self.mentions_box = mentions_box()
		self.mentions_box.user_clicked.connect( self.insert_mention )
		self.mentions_box.user_clicked.connect( self.mentions_box.hide )
		
		self.textChanged.connect( self.process )

	def set_res(self, res):
		self.mentions_box.set_res( res )

	def set_search(self, search):
		self._search = search

	def insert_mention(self, diaspora_id):
		text = self.toPlainText()
		text = text.replace(self.last_query, "@{{{}}}".format(diaspora_id), 1) # TODO this is dirty
		self.setPlainText(text)

	def process(self):
		if not self._search: return
		
		cursor = self.textCursor()
		cursor_pos = cursor.position()
		text = self.toPlainText()
		
		""" Mention search """
		# If text without whitespace is greater then 1
		if len(text.strip()) >1:
			look_back = 32 # How many character to look back
			# TODO look forward
			if len(text) < look_back:
				lookback = len(text)
			
			# If look_back character before the cursor position there is a @ char
			if '@' in text[cursor_pos-look_back:cursor_pos]:
				# Look if we need to search
				search = ""
				for char in text[cursor_pos-look_back:cursor_pos]:
					if char == '@':
						search += char # Now search is not ""
						continue
					if search and not char.strip() or char in ['#','!','$','%','^','&','*','(',')','[',']',';']: # If there is a whitespace, reset search
						# TODO just check if char is [a-zA-Z0-9\._-]+
						search = ""
					elif search: # All ok, adding next char
						search += char
						
				if len(search) >2: # including the @ we want atleast 3 characters before doing a search
					print(search) # DEBUG
					self.last_query = search
					if self.mentions_box.isHidden():

						""" Do remote search """
						people = self._search.user(search[1:]) # skip the @, TODO Thread
						if people:
							self.mentions_box.set_mentions(people)
							self.mentions_box.show()
						elif not self.mentions_box.isHidden():
							self.mentions_box.hide()

				elif not self.mentions_box.isHidden():
					self.mentions_box.hide()
			elif not self.mentions_box.isHidden():
				self.mentions_box.hide()

	def mousePressEvent(self, event):
		self.clicked.emit()
		QtWidgets.QPlainTextEdit.mousePressEvent(self, event)

class markdown_editor(QtWidgets.QFrame, Ui_markdown_editor):
	"""
	object	: list with aspect id's
	str		: markdown
	str		: photo
	object	: list with photo id's
	str		: poll question
	object	: list with poll answers """
	submit_request = pyqtSignal(object, str, str, object, str, object)
	submit_request_comment = pyqtSignal(str) # str: markdown

	diaspora_size_limit = 4194304
	def __init__(self, is_comment=False, res=None, max_width=100, parent=None):
		QtWidgets.QFrame.__init__(self, parent=parent)
		self.setupUi(self)
		
		self.max_width = max_width
		self.__collapsed = False
		self.__is_comment = is_comment
		
		if self.__is_comment:
			self.btn_share.setText("Comment")

		self.publisher_handle = None
		self.uploaded_photos = {}
		
		"""self.aspects is a list with dicts
		dict example: {'id': 123, 'name': 'Friends', 'selected': True}
		"""
		self.aspects = []
		
		""" editor """
		self.edit = edit()
		self.edit_layout.addWidget( self.edit )

		# Poll editor
		self.poll_edit = None

		""" hide photo thumb area, where thumbs appear when photo's are
		uploaded """
		self.photo_thumb_area.hide()
		
		""" preview (markdown_viewer) """
		self.markdown_viewer = markdown_viewer( res=res )
		
		sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
		sizePolicy.setHorizontalStretch(0)
		sizePolicy.setVerticalStretch(0)
		sizePolicy.setHeightForWidth(self.markdown_viewer.sizePolicy().hasHeightForWidth())
		self.markdown_viewer.setSizePolicy(sizePolicy)
		
		self.markdown_viewer.content_updated.connect( self.resize_markdown_viewer )
		
		self.preview_layout.addWidget( self.markdown_viewer, 0, QtCore.Qt.AlignTop )
		
		""" aspects_button """
		if not self.__is_comment:
			self.publisher.aspects_button = aspects_button(override_checked=False)
			self.publisher_layout.insertWidget(1, self.publisher.aspects_button)
		
		""" hooks """
		self.tabWidget.currentChanged.connect( self.tab_changed )
		self.close.clicked.connect( self.collapse )
		self.edit.clicked.connect( self.expand )

		# Edit hooks
		self.btn_bold.clicked.connect( self.bold )
		self.btn_italic.clicked.connect( self.italic )
		self.btn_header.clicked.connect( self.header )
		
		self.btn_link.clicked.connect( self.link )
		self.btn_img.clicked.connect( self.image )
		
		self.btn_pre.clicked.connect( self.pre )
		self.btn_list.clicked.connect( self.list )
		self.btn_nr_list.clicked.connect( lambda: self.list(style='nr') )
		
		self.btn_code.clicked.connect( self.code )
		self.btn_quote.clicked.connect( self.quote )

		# Publish
		self.btn_share.clicked.connect( self.publish )

		# Footer hooks
		self.btn_poll.clicked.connect( self.toggle_poll )
		if not self.__is_comment:
			self.btn_add_photo.clicked.connect( self.add_photos_dialog )
		
		""" do on init """
		self.collapse()
		
		""" TODO
		* replace '\u2029' with \n
		* when adding markdown, check if same markdown is in selected range
		* on markdown_viewer (tab_changed): thread and loading icon
		* footer functions: poll, photo and location
		"""

	def toggle_poll(self):
		if not self.poll_edit:
			self.poll_edit = poll_edit()
			self.poll_edit.ui.btn_delete_poll.clicked.connect(self.delete_poll_edit)
			self.verticalLayout.insertWidget(2, self.poll_edit)
		elif self.poll_edit.isHidden(): self.poll_edit.show()
		else: self.poll_edit.hide()

	def delete_poll_edit(self):
		self.poll_edit.ui.btn_delete_poll.clicked.disconnect(self.delete_poll_edit)
		self.poll_edit.deleteLater()
		self.poll_edit = None

	def set_res(self, res):
		self.edit.set_res( res )
		self.markdown_viewer.res = res

	def set_search(self, search): # edit needs this to search for mentions
		self.edit.set_search(search)

	""" validate markdown before posting """
	def validate(self, markdown):
		# check if total length of markdown doesn't exceed x
		# .. TODO findout x
		
		# check if used any tags
		# if so check if they are smaller then 256 chars
		tags = re.findall(r'#([\w0-9\_\-]+)', markdown)
		for tag in tags:
			if len(tag) > 255:
				return False
		return True

	def photo_upload(self, path_list, aspects_list=[]):
		# TODO thread and exceptions
		photo_ids = []
		photo_paths = []
		for img_path in path_list:
			info = QtCore.QFileInfo(img_path)
			if info.size() <= self.diaspora_size_limit: # Check file size limit
				photo_id = self.publisher_handle._photoupload(filename=img_path, aspects=aspects_list)
				photo_ids.append( photo_id )
				photo_paths.append( img_path )
			else: # file size to bigger then limit.
				print("Could not upload {0}, file to big.".format(img_path))
		return photo_ids, photo_paths

	def photos_uploaded(self, photo_ids, photo_paths):
		if photo_ids and photo_paths: # check if lists are not empty, if they are then we don't want to show the photo_thumb_area
			index = 0
			for photo_path in photo_paths:
				thumb = deletable_thumb( photo_ids[index], photo_path )
				self.uploaded_photos.update( { photo_ids[index]: thumb } )
				self.photo_thumb_layout.addWidget( self.uploaded_photos[photo_ids[index]] )
				thumb.delete_photo.connect( self.remove_photo_thumb )
				index +=1
			self.photo_thumb_area.show()

	def add_photos_dialog(self):
		""" Diaspora allows the following formats:
		*.jpeg *.jpg *.png *.gif

		TODO:
		- check for max file size
		- check for max resolution
		- client size resize?
		"""
		options = QtWidgets.QFileDialog.Options()
		options |= QtWidgets.QFileDialog.DontUseNativeDialog
		photos, _ = QtWidgets.QFileDialog.getOpenFileNames(self,"Select images", "","Images / Photo's (*.jpeg *.jpg *.png *.gif);;All (*)", options=options)
		if photos:
			photo_ids, photo_paths = self.photo_upload(photos, aspects_list=self.publisher.aspects_button.selected_aspects())
			self.photos_uploaded( photo_ids, photo_paths )
			print(photos) # DEBUG

	def input(self, title="", label=""):
		text, state = QtWidgets.QInputDialog.getText(self, title, label)
		if state: return text
		return None

	def set_publisher(self, publisher): self.publisher_handle = publisher

	def remove_photo_thumb(self, photo_id, local_only=False):
		self.uploaded_photos[photo_id].delete_photo.disconnect()
		self.uploaded_photos[photo_id].deleteLater()
		self.uploaded_photos.pop(photo_id)

		# TODO thread
		if not local_only:
			self.publisher_handle._deletephoto(photo_id)
			print("DELETED PHOTO ID {} FROM REMOTE!".format(photo_id)) # DEBUG

		if not self.uploaded_photos: self.photo_thumb_area.hide()

	def clear_thumbs(self, local_only=True):
		photo_ids = []
		for photo_id in self.uploaded_photos: photo_ids.append(photo_id)
		for photo_id in photo_ids: self.remove_photo_thumb(photo_id, local_only=local_only)

	def publish(self):
		# .. start loading icon
		markdown = self.markdown()
		if len(markdown.strip()):
			if not self.validate(markdown):
				# TODO print graphical message
				print("Tag to long") # DEBUG
				return

			if self.__is_comment:
				# TODO Photo's
				self.submit_request_comment.emit(markdown)
			else:
				photo = '' # TODO photo
				photos = []
				if self.uploaded_photos:
					for photo_id in self.uploaded_photos:
						photos.append(photo_id)

				poll_question = None
				poll_answers = None
				if self.poll_edit:
					poll_data = self.poll_edit.data()
					if poll_data:
						poll_question = poll_data[0]
						poll_answers = poll_data[1]
				self.submit_request.emit(self.publisher.aspects_button.selected_aspects(), markdown, photo, photos, poll_question, poll_answers)

			# Disable share button.
			self.btn_share.setEnabled(False)

	def not_published(self):
		self.btn_share.setEnabled(True)
		# .. stop loading icon

	def published(self): # slot for when post thread finished
		self.btn_share.setEnabled(True)
		self.edit.setPlainText("")
		if self.uploaded_photos: self.clear_thumbs() # Clear uploaded photo thumbs.
		if self.poll_edit: self.delete_poll_edit()
		self.collapse()

	def markdown(self): # get markdown
		markdown = self.edit.toPlainText()
		return markdown.replace('\u2029', '\n')

	def set_aspects(self, aspects):
		self.aspects = aspects
		self.publisher.aspects_button.set_aspects(aspects)
		self.publisher.aspects_button.update_aspects()

	def set_max_width(self, max_width):
		self.max_width = max_width
		self.setMaximumWidth( max_width )
		self.resize_markdown_viewer()

	def tab_changed(self, index):
		if index: # then index is 1, so: preview
			# TODO loading icon and thread (for getting remote images)
			self.markdown_viewer.set_markdown( self.markdown() )
			self.resize_markdown_viewer()

	def resize_markdown_viewer(self):
		self.markdown_viewer.resize_content( self.max_width-100, collapse=False )

	def set_avatar(self, path):
		if path:
			pixmap = QtGui.QPixmap(path)
			if pixmap:
				pixmap.scaledToWidth(50)
				pixmap.scaledToHeight(50)
				self.avatar.setMaximumSize(QtCore.QSize(50, 50))
				self.avatar.setMinimumSize(QtCore.QSize(50, 50))
				self.avatar.setPixmap(pixmap)

	def expand(self):
		if self.__collapsed:
			self.button_groups.show()
			if not self.__is_comment: self.footer.show()
			self.tabWidget.tabBar().show()
			self.publisher.show()
			self.setMinimumHeight(0)
			self.setMaximumHeight(16777215)
			self.__collapsed = False

	def collapse(self):
		self.tabWidget.setCurrentIndex(0)
		self.button_groups.hide()
		self.footer.hide()
		self.publisher.hide()
		self.tabWidget.tabBar().hide()
		self.setMinimumHeight(80)
		self.setMaximumHeight(80)
		self.adjustSize()
		self.photo_thumb_area.hide()
		self.__collapsed = True

	def list(self, style=None):
		start, end, text = self.get_selected_text_range()
		if not text:
			cursor = self.edit.textCursor()
			start = self.get_line_start( cursor.position() )
			self.set_cursor_position(start)
			if style == 'nr':
				self.edit.insertPlainText("1. ")
			else:
				self.edit.insertPlainText("- ")
		else:
			pos = start
			count = 1
			text += '\u2029'
			for char in text:
				#print(repr(char)) # DEBUG
				if char == '\u2029': # if white space
					self.set_cursor_position( self.get_line_start( pos ) )
					if style == 'nr':
						bullet = "{}. ".format(count)
						self.edit.insertPlainText( bullet )
						count +=1
						end += len(bullet)
					else:
						self.edit.insertPlainText("- ")
						end += 2
				pos +=1
			self.select_text(start, end)

	def bold(self):
		start, end, text = self.get_selected_text_range()
		if not text:
			self.edit.insertPlainText("** **")
		else:
			self.set_cursor_position(start)
			self.edit.insertPlainText("**")
			self.set_cursor_position(end+2)
			self.edit.insertPlainText("**")
			self.select_text(start, end+4)
			
	def code(self):
		start, end, text = self.get_selected_text_range()
		if not text:
			self.edit.insertPlainText("\u2029````\u2029Code here..\u2029````\u2029")
		else:
			self.set_cursor_position(start)
			self.edit.insertPlainText("\u2029````\u2029")
			self.set_cursor_position(end+6)
			self.edit.insertPlainText("\u2029````\u2029")
			self.select_text(start, end+12)

	def quote(self, style=None):
		start, end, text = self.get_selected_text_range()
		if not text:
			cursor = self.edit.textCursor()
			start = self.get_line_start( cursor.position() )
			self.set_cursor_position(start)
			self.edit.insertPlainText("> ")
		else:
			pos = start
			text += '\u2029'
			for char in text:
				if char == '\u2029': # if white space
					self.set_cursor_position( self.get_line_start( pos ) )
					self.edit.insertPlainText("> ")
					end += 2
				pos +=1
			self.select_text(start, end)

	def link(self):
		start, end, text = self.get_selected_text_range()
		link = self.input(title="Please enter url to link to", label="url: ") # TODO validate url and ask for alt_text + title
		if not text:
			self.edit.insertPlainText("[displayed text here]({} \"alt text\")".format( link ))
		else:
			self.set_cursor_position(start)
			self.edit.insertPlainText("[")
			self.set_cursor_position(end+1)
			self.edit.insertPlainText("]({} \"alt text\")".format( link ))
			self.set_cursor_position(end)
	
	def image(self):
		start, end, text = self.get_selected_text_range()
		link = self.input(title="Please enter image url", label="image url: ") # TODO validate url and ask for alt_text + title (maybe link too?)
		if not text:
			self.edit.insertPlainText("![image text here]({} \"alt text\")".format( link ))
		else:
			self.set_cursor_position(start)
			self.edit.insertPlainText("![")
			self.set_cursor_position(end+1)
			self.edit.insertPlainText("]({} \"alt text\")".format( link ))
			self.set_cursor_position(end)
		
	def italic(self):
		start, end, text = self.get_selected_text_range()
		if not text:
			self.edit.insertPlainText("* *")
		else:
			self.set_cursor_position(start)
			self.edit.insertPlainText("*")
			self.set_cursor_position(end+1)
			self.edit.insertPlainText("*")
			self.select_text(start, end+2)

	def header(self):
		cursor = self.edit.textCursor()
		start = self.get_line_start( cursor.position() )
		self.set_cursor_position(start)
		self.edit.insertPlainText("# ")
		
	def pre(self):
		cursor = self.edit.textCursor()
		start = self.get_line_start( cursor.position() )
		self.set_cursor_position(start)
		self.edit.insertPlainText("    ")

	def get_selected_text_range(self):
		cursor = self.edit.textCursor()
		if cursor.hasSelection():
			start = cursor.selectionStart()
			end = cursor.selectionEnd()
			return (start, end, cursor.selectedText())
		return [0,0,None] 

	def get_line_start(self, pos):
		cursor = self.edit.textCursor()
		cursor.setPosition(pos)
		cursor.movePosition(QtGui.QTextCursor.StartOfLine)
		return cursor.position()
	
	def get_line_end(self, pos):
		cursor = self.edit.textCursor()
		cursor.setPosition(pos)
		cursor.movePosition(QtGui.QTextCursor.EndOfLine)
		return cursor.position()

	def set_cursor_position(self, pos):
		cursor = self.edit.textCursor()
		cursor.setPosition(pos)
		self.edit.setTextCursor(cursor)
		
	def select_text(self, start, end):
		cursor = self.edit.textCursor()
		cursor.setPosition(start)
		cursor.setPosition(end, QtGui.QTextCursor.KeepAnchor)
		
		self.edit.setTextCursor(cursor)
