"""
This file is part of LionsTooth.

LionsTooth is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

LionsTooth is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LionsTooth.  If not, see <http://www.gnu.org/licenses/>.
"""
from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtCore import pyqtSignal

class message(QtWidgets.QWidget):
	clicked = pyqtSignal()
	def __init__(self, parent=None):
		QtWidgets.QWidget.__init__(self, parent)
		self.background = QtGui.QColor(255, 0, 0)
		
		""" layout """
		self.layout = QtWidgets.QVBoxLayout(self)
		self.layout.setContentsMargins(12, 12, 12, 0)
		self.layout.setSpacing(0)
		
		""" title """
		self.title = QtWidgets.QLabel(self)
		self.title.setText("<h3>This is a message</h3>")
		self.title.setStyleSheet("background-color:#3f6934;\n"
		"color:#ffffff;\n"
		"text-align: center;\n");
		self.title.setAlignment(QtCore.Qt.AlignCenter);

		""" msg container """
		self.msg_container = QtWidgets.QTextEdit(self)

		self.msg_container.setFrameShape(QtWidgets.QFrame.NoFrame)
		self.msg_container.document = QtGui.QTextDocument()
		
		self.msg_container.setReadOnly(True)
		self.msg_container.setWordWrapMode(QtGui.QTextOption.WrapAtWordBoundaryOrAnywhere)
		self.msg_container.setDocument( self.msg_container.document )
		self.msg_container.setTextInteractionFlags(QtCore.Qt.NoTextInteraction)
		self.msg_container.setObjectName("msg_container")
		self.msg_container.setStyleSheet("QTextEdit#msg_container {\n"
		"	background-color: rgb(63, 99, 54);\n"
		"}\n");
		
		""" self """
		sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
		sizePolicy.setHorizontalStretch(0)
		sizePolicy.setVerticalStretch(0)
		sizePolicy.setHeightForWidth(self.sizePolicy().hasHeightForWidth())
		self.setSizePolicy(sizePolicy)
		
		""" close button (well.. label) """
		self.close_button = QtWidgets.QPushButton(self)
		self.close_button.setText("Close")
		self.close_button.setObjectName("close_button")
		self.close_button.setMaximumHeight(20)
		
		self.close_button.setStyleSheet("QPushButton#close_button {\n"
		"	background-color: rgb(63, 99, 54);\n"
		"	font-weight: bold;"
		"}\n");
		
		self.layout.addWidget(self.title)
		self.layout.addWidget(self.msg_container)
		self.layout.addWidget(self.close_button)
		
		self.width = 0
		self.height = 0
		self.size = self.size()
		self.hide()

		# Hooks
		self.close_button.clicked.connect(self.hide_msg)
	
	def update_size(self, size, margins):
		self.width = size.width()
		self.width -= margins.left()
		self.width -= margins.right()
		self.move(
			margins.left(),
			margins.top()
		)
		self.setMinimumWidth( self.width )
		self.setMaximumWidth( self.width )
	
	def print_msg(self, title="", msg=""):
		if not msg:
			msg = "Message is not set or None"
		
		self.msg_container.document.setHtml( "<b>{}</b>".format(msg) )
		self.msg_container.document.setTextWidth( self.width )
		self.msg_container.document.setDocumentMargin(12)
		
		self.title.setText("<h3>{}</h3>".format(title))
		
		size = self.msg_container.document.size()
		self.height = size.height()
		self.height += self.close_button.size().height()
		self.height += self.title.size().height()

		self.msg_container.setMinimumHeight( self.height )
		
		self.setMinimumHeight( self.height )
		self.setMaximumHeight( self.height + 10)

		self.show()

	def hide_msg(self):
		self.hide()
	
	def mousePressEvent(self, event):
		self.clicked.emit()
