"""
This file is part of LionsTooth.

LionsTooth is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

LionsTooth is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LionsTooth.  If not, see <http://www.gnu.org/licenses/>.
"""
from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtCore import pyqtSignal

from design.ui_notification import Ui_notification
from classes.formatter import tago, f_timestamp
from widgets.shared.html_content import html_content

class notification(QtWidgets.QFrame):
	#new = pyqtSignal(bool)
	seen_clicked = pyqtSignal(bool)# bool: seen or unseen
	anchor_clicked = pyqtSignal(str) # href
	def __init__(self, n, max_width=None, parent=None):
		QtWidgets.QFrame.__init__(self, parent)

		self.ui = Ui_notification()
		self.ui.setupUi(self)
		
		self.n = n
		self.max_width = max_width
		
		self.avatar_url = None
		self.people = {} # Avatar is from the first one.
		
		self.css = {}
		self.css['unseen'] = """QFrame{\n
			background-color: rgb(96, 107, 51);\n
		}\n
		QFrame#notification{\n
			border-bottom: 1px solid #2c2c2c;\n
		}"""
		self.css['seen'] = """QFrame{\n
			background-color: rgb(119, 134, 64, 0);\n
		}\n
		QFrame#notification{\n
			border-bottom: 1px solid #2c2c2c;\m
		}"""
		
		self.icon = {}
		self.icon['eye_normal'] = QtGui.QIcon()
		self.icon['eye_active'] = QtGui.QIcon()
		self.icon['eye_normal'].addPixmap(QtGui.QPixmap(":/icons/icons/eye_48_grey.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
		self.icon['eye_active'].addPixmap(QtGui.QPixmap(":/icons/icons/eye_48_white.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
		
		""" seen button """
		self.checked = not bool(n['unread']) # if self.checked is True it means it has been seen
		self.seen( self.checked )
		self.ui.btn_toggle.clicked.connect( self.toggle )
		
		""" time ago """
		self.ui.lbl_tago.setText( tago( n['updated_at'] ) )
		self.ui.lbl_tago.setToolTip( f_timestamp( n['updated_at'] ) )
		
		""" notificaton itself """
		self.ui.notification = html_content()
		self.ui.horizontalLayout.insertWidget(0, self.ui.notification)
		self.ui.notification.anchor_clicked.connect( self.anchor_clicked )
		if not self.checked: # if unseen
			self.ui.notification.anchor_clicked.connect( self.clicked ) # sets to seen

	def clicked(self, link):
		if not self.checked: # if unseen
			_type, other = link.lstrip('/').split('/', 1)
			if _type == 'posts':
				self.ui.notification.anchor_clicked.disconnect( self.clicked ) # not needed anymore
				self.seen()
				self.seen_clicked.emit(True)  # let the header know it should update it's stats

	def toggle(self):
		if self.checked: # if seen set to unseen
			self.checked = False
			self.seen(False)
			self.seen_clicked.emit(False) # will emit False when set from seen to unseen
			self.n.mark(True) # TODO thread
			self.ui.notification.anchor_clicked.connect( self.clicked ) # sets to seen
		else: # if unseen set to seen
			self.checked = True
			self.seen(True)
			self.seen_clicked.emit(True) # will emit True when set from unseen to seen
			self.n.mark(False) # TODO thread

	def seen(self, bool=True):
		if bool:
			self.ui.btn_toggle.setIcon( self.icon['eye_normal'] )
			self.setStyleSheet( self.css['seen'] )
		else:
			self.ui.btn_toggle.setIcon( self.icon['eye_active'] )
			self.setStyleSheet( self.css['unseen'] )

	def set_text(self, text):
		self.ui.notification.update_html_content( "<html><head/><body><p>{}</p></body></html>".format(text) )
		self.ui.notification.resize_content( self.max_width-120 )

	def update_avatar(self, path):
		if path:
			pixmap = QtGui.QPixmap(path)
			if pixmap:
				pixmap.scaledToWidth(35)
				pixmap.scaledToHeight(35)
				self.ui.avatar.setMaximumSize(QtCore.QSize(35, 35))
				self.ui.avatar.setMinimumSize(QtCore.QSize(35, 35))

				self.ui.avatar.setPixmap(pixmap)
