"""
This file is part of LionsTooth.

LionsTooth is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

LionsTooth is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LionsTooth.  If not, see <http://www.gnu.org/licenses/>.
"""
from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtCore import pyqtSignal

from design.ui_notifications import Ui_notifications
from widgets.notification import notification

class notifications(QtWidgets.QWidget):
	request_more = pyqtSignal()
	request_update = pyqtSignal()
	notification_toggled = pyqtSignal(bool)# True if a notification is set to unseen, False if a notification is set to seen. So +1 or -1
	anchor_clicked = pyqtSignal(str) # href
	def __init__(self, parent=None):
		QtWidgets.QWidget.__init__(self, parent)

		self.ui = Ui_notifications()
		self.ui.setupUi(self)
		
		self.notifications = {}
		
		self.width = 0
		self.height = 400
		self.max_height = 0
		self.top_margin = 75 # TODO this is the header height
		self.size = self.size()
		self.hide()
		
		self.setWindowFlags(QtCore.Qt.FramelessWindowHint | QtCore.Qt.Popup) # This is needed for: if clicked outside of this widget then close.

		""" Hooks """
		self.ui.scrollArea.verticalScrollBar().valueChanged.connect( self.scroll_changed ) # Request more notifications
		self.ui.btn_load_more.clicked.connect( self.load_more_button ) # TODO hide this again when scrolling appears
		self.ui.btn_mark_all.clicked.connect( self.mark_all )
		self.anchor_clicked.connect( self.hide ) # hide when a link is clicked

	def load_more_button(self):
		if self.ui.scrollArea.verticalScrollBar().maximum():
			self.ui.btn_load_more.clicked.disconnect()
			self.ui.btn_load_more.hide()
		self.request_more.emit()

	def update_notifications(self, new_notifications):
		pass

	def scroll_changed(self, value):
		if not self.ui.btn_load_more.isHidden():
			self.ui.btn_load_more.hide()
			self.ui.btn_load_more.clicked.disconnect()

		one_percent =  self.ui.scrollArea.verticalScrollBar().maximum() / 100
		if one_percent and round(value / one_percent) > 99:
			self.request_more.emit()
			print("request more notifications..")

	def mark_all(self):
		for id in self.notifications:
			if not self.notifications[id].checked:
				self.notifications[id].toggle()

	def add(self, n):
		if n.id not in self.notifications:
			self.notifications.update( { n.id: notification(n) } )
			self.notifications[n.id].seen_clicked.connect( self.notification_toggled )	# connect seen toggled
			self.ui.content.addWidget( self.notifications[n.id] )

	def insert(self, n, index=0):
		if n.id not in self.notifications:
			self.notifications.update( { n.id: notification(n, max_width=self.width) } )

			self.notifications[n.id].anchor_clicked.connect( self.anchor_clicked ) # connect anchor clicked
			self.notifications[n.id].seen_clicked.connect( self.notification_toggled )	# connect seen toggled

			self.notifications[n.id].ui.notification.resize_content(  self.width - 100 )
			self.ui.content.insertWidget(index, self.notifications[n.id] )

	def resize_notifications(self, max_width=None):
		if not max_width:
			max_width = self.width - 100 # min avatar and margins
		for id in self.notifications:
			self.notifications[id].ui.notification.resize_content( max_width )

	def update_size(self, size, margins):
		self.max_height = size.height()
		if self.height < self.max_height:
			self.setMinimumHeight( self.height )
		self.setMaximumHeight( self.max_height )
		
		self.width = size.width()
		self.width -= margins.left()
		self.width -= margins.right()
		self.width -= 6*2
		self.move(
			margins.left(),
			margins.top() + self.top_margin # TODO
		)
		self.setMinimumWidth( self.width )
		self.setMaximumWidth( self.width )
		self.resize_notifications()
