# -*- coding: utf-8 -*-
"""
This file is part of LionsTooth.

LionsTooth is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

LionsTooth is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LionsTooth.  If not, see <http://www.gnu.org/licenses/>.
"""
from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtCore import pyqtSignal, pyqtSlot

from design.ui_open_graph import Ui_open_graph

from classes.resolver import img_worker

class open_graph(QtWidgets.QFrame):
	resized = pyqtSignal()
	anchor_clicked = pyqtSignal(str) # link
	def __init__(self, open_graph_cache, res, parent=None):
		QtWidgets.QWidget.__init__(self, parent=parent)
		self.ui = Ui_open_graph()
		self.ui.setupUi(self)
		self.res = res
		
		self.max_img_width = 100
		
		self.setStyleSheet("background-color: rgb(26, 37, 24);\n"
			"color: rgb(208, 221, 255);")
		self.anchor_css = "style=\"color: #8ab854;\""

		#print(open_graph_cache) # DEBUG
		self.set_title( open_graph_cache['title'], url=open_graph_cache['url'] )
		self.set_description( open_graph_cache['description'] )
		if open_graph_cache['image']:
			worker = img_worker(self.res, url=open_graph_cache['image'])
			worker.signals.finished.connect(self.set_image)
			self.res.thread_pool.start(worker)

		# Anchor hook
		self.ui.title.linkActivated.connect( self.anchor_clicked )

	def set_title(self, title, url=None):
		if url:
			self.ui.title.setText("<a href=\"{0}\" {1}><b>{2}</b></a>".format(url, self.anchor_css, title))
		else:
			self.ui.title.setText("<b>{0}</b>".format(title))
		
	def set_description(self, description):
		self.ui.description.setText(description)
		
	def set_image(self, path):
		self.pixmap = QtGui.QPixmap(path)
		self.pixmap = self.pixmap.scaledToWidth( self.max_img_width );

		if self.pixmap:
			self.ui.image.setPixmap(self.pixmap)
			self.resized.emit()
		else:
			print("open_graph failed to set img: {}".format(path))
		
