# -*- coding: utf-8 -*-
"""
This file is part of LionsTooth.

LionsTooth is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

LionsTooth is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LionsTooth.  If not, see <http://www.gnu.org/licenses/>.
"""
from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtCore import pyqtSignal, pyqtSlot
from design.ui_photo_browser import Ui_photo_browser
from widgets.shared.author_name import author_name

from widgets.markdown_viewer import image

""" images """
from classes.resolver import resolver

""" Time formatting """
from classes.formatter import tago, f_timestamp

""" Img downloading / caching """
from classes.resolver import img_worker

class thumb(QtWidgets.QLabel):
	clicked = pyqtSignal(int)
	delete_photo = pyqtSignal(int)
	def __init__(self, index, own_profile=False, parent=None):
		QtWidgets.QLabel.__init__(self, parent)
		self.index = index
		self.own_profile = own_profile
		self.setMaximumSize(QtCore.QSize(16777215, 100))
		self.setFrameShape(QtWidgets.QFrame.StyledPanel)
		
		sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
		#sizePolicy.setHorizontalStretch(0)
		#sizePolicy.setVerticalStretch(0)
		sizePolicy.setWidthForHeight(self.sizePolicy().hasWidthForHeight())
		self.setSizePolicy(sizePolicy)

		# Delete button
		if own_profile:
			self.delete_button = QtWidgets.QPushButton(self)
			icon = QtGui.QIcon()
			icon.addPixmap(QtGui.QPixmap(":/icons/icons/delete_thumb.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
			self.delete_button.setIcon(icon)
			self.delete_button.setFlat(True)
			self.delete_button.setIconSize( QtCore.QSize(16, 16) )
			self.delete_button.setToolTip("Delete this photo.")

			self.delete_button.hide()

			# Delete button hook
			self.delete_button.clicked.connect( self._delete_photo )

	def _delete_photo(self): self.delete_photo.emit( self.index )

	def enterEvent(self, event):
		if self.own_profile:
			self.delete_button.show()
			x = self.size().width()
			x -= self.delete_button.size().width()
			y = self.size().height()
			y -= self.delete_button.size().height()
			self.delete_button.move(x,y)
		
	def leaveEvent(self, event):
		if self.own_profile: self.delete_button.hide()

	def mousePressEvent(self, event):
		self.clicked.emit(self.index)

class photo_browser(QtWidgets.QFrame):
	resized = pyqtSignal()
	author_name_clicked = pyqtSignal(str, str) # guid, diaspora_id
	delete_photo_request = pyqtSignal(int) # photo id
	def __init__(self, photo_data, res, max_width=None, own_profile=False, parent=None):
		QtWidgets.QWidget.__init__(self, parent=parent)
		
		self.ui = Ui_photo_browser()
		self.ui.setupUi(self)
		
		""" the photo itself """
		self.ui.photo = image()
		sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Maximum)
		self.ui.photo.setSizePolicy(sizePolicy)
		self.ui.view_layout.insertWidget(1, self.ui.photo, QtCore.Qt.AlignHCenter)
		
		self.photo_data = photo_data
		self.photo_count = len(self.photo_data)
		self.pixmap = None
		self.image_path = None
		
		self.setStyleSheet("background-color: rgb(26, 37, 24);\n"
			"color: rgb(208, 221, 255);")
		
		self.max_width = max_width
		self.original_width = None
		
		self.res = res
		self.own_profile = own_profile
		self.current_index = 0
		self.thumbs = {} # key: index,  value: widget

		if not own_profile: self.ui.btn_delete.hide()
		else: self.ui.btn_delete.clicked.connect( self.delete_photo )

		""" author name """
		self.ui.lbl_author = author_name()
		self.ui.lbl_author.clicked.connect( self.author_name_clicked_func )
		self.ui.verticalLayout_2.insertWidget(0, self.ui.lbl_author)

		self.ui.btn_left.setEnabled(False)
		if self.photo_count == 1:
			self.ui.thumb_scroll.hide()
			self.ui.btn_right.hide()
			self.ui.btn_left.hide()
		else:
			self.ui.btn_left.clicked.connect(self.previous_photo)
			self.ui.btn_right.clicked.connect(self.next_photo)

			""" get thumbs """
			for index in range(self.photo_count):
				self.add_thumb(index) # place holder
				worker = img_worker(self.res, url=self.photo_data[index]['sizes']['medium'], index=index)
				worker.signals.finished_index.connect(self.update_thumb)
				self.res.thread_pool.start(worker)
		self.set_data()
	
	def author_name_clicked_func(self):
		guid = self.photo_data[self.current_index]['author']['guid']
		diaspora_id = self.photo_data[self.current_index]['author']['diaspora_id']
		self.author_name_clicked.emit(guid, diaspora_id)

	def delete_photo(self, index=None): # deletes current photo
		sure = QtWidgets.QMessageBox.question(self, "Delete photo", "Are you sure you want to delete this photo?", QtWidgets.QMessageBox.Yes, QtWidgets.QMessageBox.No)
		if sure == QtWidgets.QMessageBox.Yes:
			if index:
				self.delete_photo_request.emit( self.photo_data[index]['id'] )
			else: # current index
				print("delete: {}".format( self.photo_data[self.current_index] ))
				self.delete_photo_request.emit( self.photo_data[self.current_index]['id'] )

	def photo_deleted(self, photo_id):
		# TODO update people object (emit signal)
		
		# Get index of photo_id
		index = 0
		for photo in self.photo_data:
			if photo['id'] == photo_id: break
			index += 1
		
		# Go to next or previous photo
		# Previous if available
		# else next if available
		if self.current_index == index:
			if index > 0: # there is a previous index
				self.previous_photo()
			elif index < self.photo_count: # no previous index, check on next index
				self.next_photo()
			else: # no photo's left
				pass #TODO

		# Remove related thumb
		# Disconnect thumb
		self.thumbs[index].clicked.disconnect()
		if self.own_profile: self.thumbs[index].delete_photo.disconnect()
		self.thumbs[index].deleteLater()
		"""
		Do not pop, else we need to change every thumb (widget) it's index
		TODO use photo id's instead of index.
		TODO update pofile information
		"""
		self.thumbs[index] = None
		#self.thumbs.pop( index )

	def next_photo(self):
		for index in range(self.current_index+1, len(self.thumbs)):
			if self.thumbs[index]:
				self.current_index = index
				break
		
		if self.current_index == self.photo_count-1:
			self.ui.btn_right.setEnabled(False)
		if not self.ui.btn_left.isEnabled():
			self.ui.btn_left.setEnabled(True)
		self.set_data()

	def previous_photo(self):
		for index in reversed(range(0, self.current_index)):
			if self.thumbs[index]:
				self.current_index = index
				break
		
		if self.current_index == 0:
			self.ui.btn_left.setEnabled(False)
		if not self.ui.btn_right.isEnabled():
			self.ui.btn_right.setEnabled(True)
		self.set_data()

	def update_thumb(self, index, path):
		item = self.ui.horizontalLayout.itemAt(index+1) # +1 for the left spacer
		widget = item.widget()
		if widget:
			pixmap = QtGui.QPixmap(path)
			if pixmap:
				widget.setPixmap( pixmap.scaled(16777215,80,QtCore.Qt.KeepAspectRatio) )
	
	def add_thumb(self, index):
		thumb_widget = thumb(index=index, own_profile=self.own_profile, parent=self.ui.scrollAreaWidgetContents)
		thumb_widget.clicked.connect(self.thumb_clicked)
		if self.own_profile: thumb_widget.delete_photo.connect( self.delete_photo )
		self.thumbs.update( { index : thumb_widget } )

		layout_count = self.ui.horizontalLayout.count()

		self.ui.horizontalLayout.insertWidget(self.ui.horizontalLayout.count()-1, self.thumbs[index]) # -1 because we want the spacer after

	def thumb_clicked(self, index):
		self.current_index = index
		
		if self.current_index == 0: # disable left
			self.ui.btn_left.setEnabled(False)
			
			if not self.ui.btn_right.isEnabled() and self.current_index != self.photo_count-1:
				self.ui.btn_right.setEnabled(True)
		
		elif self.current_index == self.photo_count-1: # disable right
			self.ui.btn_right.setEnabled(False)
			
			if not self.ui.btn_left.isEnabled() and self.current_index != 0:
				self.ui.btn_left.setEnabled(True)
				
		else: # both should be enabled
			if not self.ui.btn_right.isEnabled():
				self.ui.btn_right.setEnabled(True)
			if not self.ui.btn_left.isEnabled():
				self.ui.btn_left.setEnabled(True)
				
		self.set_data()

	def update_avatar(self, path):
		pixmap = QtGui.QPixmap(path)
		if pixmap:
			pixmap.scaledToWidth(35)
			pixmap.scaledToHeight(35)
			self.ui.avatar.setPixmap(pixmap)

	def set_photo(self, img_path):
		self.ui.photo.update_image(img_path)
		if self.max_width: self.ui.photo.rescale( max_width=self.max_width )
		else: self.adjustSize()
		
	def resize_photo(self, max_width=None):
		self.max_width = max_width
		self.ui.photo.rescale( max_width=max_width )
		self.ui.thumb_scroll.setMaximumWidth( max_width )

	def set_data(self): # self.current_index as reference
		""" photo """
		anchor_css = "style=\"color: #fff;\""
		for size, url in self.photo_data[self.current_index]['sizes'].items():
			if size == 'small':
				self.ui.lbl_photo_small.setText( "<a href=\"{}\" {}>small</a>".format(url, anchor_css) )
			elif size == 'medium':
				self.ui.lbl_photo_medium.setText( "<a href=\"{}\" {}>medium</a>".format(url, anchor_css) )
			elif size == 'large':
				self.ui.lbl_photo_large.setText( "<a href=\"{}\" {}>large</a>".format(url, anchor_css) )
		
		""" photo thread """		
		worker = img_worker(self.res, url=self.photo_data[self.current_index]['sizes']['large'])
		worker.signals.finished.connect(self.set_photo)
		self.res.thread_pool.start(worker)

		""" avatar thread """
		worker = img_worker(self.res, url=self.photo_data[self.current_index]['author']['avatar']['small'])
		worker.signals.finished.connect(self.update_avatar)
		self.res.thread_pool.start(worker)
		
		""" set author details """
		self.ui.lbl_author.set_text( self.photo_data[self.current_index]['author']['name'] )
		self.ui.lbl_created_at.setText( tago(self.photo_data[self.current_index]['created_at']) )
		self.ui.lbl_created_at.setToolTip( f_timestamp(self.photo_data[self.current_index]['created_at']) )		
