# -*- coding: utf-8 -*-
"""
This file is part of LionsTooth.

LionsTooth is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

LionsTooth is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LionsTooth.  If not, see <http://www.gnu.org/licenses/>.
"""

from PyQt5 import QtCore, QtGui, QtWidgets
import csv
import os
from datetime import datetime, timedelta
from time import time, mktime

from design.ui_podlist import Ui_podlist

""" updating podlist """
import urllib3 # maybe lib requests is enough?
import certifi # https
import re

class podlist(QtWidgets.QWidget):
	def __init__(self, parent=None):
		QtWidgets.QWidget.__init__(self, parent=parent)
		
		self.podlist = Ui_podlist()
		self.podlist.setupUi(self)
		
		self.podfile = './cache/podlist.csv'
		if os.path.isfile(self.podfile):
			tago = datetime.now() - datetime.fromtimestamp( os.path.getmtime(self.podfile) )
			hours = round( tago.total_seconds() / 3600 )
			print("Podlist updated {} hours ago".format(hours))
			if hours > 23:
				print("should update podlist.")
				self.update_podlist()
			self.fill_table()
		else:
			print("no podlist yet")
			
		
		""" HOOKS """
		#self.tbl_podlist.doubleClicked.connect(self.doubl)
		self.podlist.btn_refresh_podlist.clicked.connect(self.refresh)

	# TODO 
	def doubl(self, la):
		txt = self.tbl_podlist.item(la.row(), 0).text()
		print("klik klik {}".format(txt))
		return True

	def refresh(self): # update and refill table
		self.update_podlist()
		self.fill_table()

	def update_podlist(self):
		print("updating podlist")
		patterns = dict()
		patterns.update({ 'tbody'					: "\s+<tbody>.*?<\/tbody>" })
		patterns.update({ 'tr'						: "<tr>.*?<\/tr>" })
		patterns.update({ 'pod_url'					: "<a class=\"text-success url\" target=\"_self\" href=\"\/go.php\?domain=([a-zA-Z0-9.-]+)\">([a-zA-Z0-9.-]+)<\/a>" })
		patterns.update({ 'uptime'					: "<td>([0-9.]+)%<\/td>"})
		patterns.update({ 'active_users'			: "<td data-toggle=\"tooltip\" data-placement=\"bottom\" title=\"Active users six months: ([0-9]+), Active users one month: ([0-9]+)\">([0-9]+)<\/td>"})
		patterns.update({ 'location'				: "<td (?:class=\"text-success\" )?data-toggle=\"tooltip\" data-placement=\"bottom\" title=\"City: ([a-zA-Z0-9, :\/]+)\">(?:<b>)?([A-Z]+)(?:<\/b>)?<\/td>"})
		patterns.update({ 'services'				: "smlogo-([a-z]+)"})

		http = urllib3.PoolManager(cert_reqs='CERT_REQUIRED', ca_certs=certifi.where())
		html = http.request('GET', 'https://podupti.me/').data.decode('utf-8')

		tbody = re.search(patterns['tbody'],html,re.MULTILINE|re.I|re.DOTALL).group(0)
		pods = re.findall(patterns['tr'], tbody, re.MULTILINE|re.I|re.DOTALL)

		with open(self.podfile, 'w', newline='') as csvfile:
			spamwriter = csv.writer(csvfile, delimiter=';', quotechar='|', quoting=csv.QUOTE_MINIMAL)

			for pod in pods:
				pod_url = re.search(patterns['pod_url'],pod)
				
				pod_uptime = re.search(patterns['uptime'],pod)
				if not pod_uptime:
					pod_uptime = "X"
				else:
					pod_uptime = pod_uptime.group(1)
				
				pod_active_users = re.search(patterns['active_users'],pod)
				if not pod_active_users:
					pod_active_users = ""
				else:
					pod_active_users = pod_active_users.group(3)
					
				pod_location = re.search(patterns['location'],pod)
				if not pod_location:
					pod_location = "X"
				else:
					pod_location = pod_location.group(2)

				pod_services = re.findall(patterns['services'], pod, re.MULTILINE)

				if pod_url:
					service_str = ""
					if pod_services:
						for service in pod_services:
							service_str += service + ", "
					spamwriter.writerow([pod_url.group(1), pod_uptime, pod_active_users, pod_location, service_str])
				else:
					print("not good: ".format(pod))
		print("done updating")

	def clear_table(self):
		while self.podlist.tbl_podlist.rowCount() > 0:
			self.podlist.tbl_podlist.removeRow(0)

	def fill_table(self):
		self.clear_table()
		with open(self.podfile, newline='') as csvfile:
			spamreader = csv.reader(csvfile, delimiter=';', quotechar='|', quoting=csv.QUOTE_MINIMAL)
			for pod_url, pod_uptime, pod_active_users, pod_location, service_str in spamreader:
				#print("{}%\t{}\t{}\t{}\t\t\t{}".format(pod_uptime, pod_active_users, pod_location, pod_url, service_str));
				row_index = self.podlist.tbl_podlist.rowCount()
				self.podlist.tbl_podlist.insertRow( row_index )
				self.podlist.tbl_podlist.setItem(row_index, 0, QtWidgets.QTableWidgetItem( pod_url ))
				self.podlist.tbl_podlist.setItem(row_index, 1, QtWidgets.QTableWidgetItem( pod_location ))
				self.podlist.tbl_podlist.setItem(row_index, 2, QtWidgets.QTableWidgetItem( pod_active_users ))
				self.podlist.tbl_podlist.setItem(row_index, 3, QtWidgets.QTableWidgetItem( "{}%".format(pod_uptime) ))
				self.podlist.tbl_podlist.setItem(row_index, 4, QtWidgets.QTableWidgetItem( service_str ))
		self.podlist.tbl_podlist.resizeColumnToContents(0)
		return True
