# -*- coding: utf-8 -*-
"""
This file is part of LionsTooth.

LionsTooth is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

LionsTooth is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LionsTooth.  If not, see <http://www.gnu.org/licenses/>.
"""
from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtCore import pyqtSignal, QObject
from design.ui_post_controls import Ui_post_controls

class signals(QObject):
	subscribe = pyqtSignal(bool) # True for subscript, False for unsubscribe
	report = pyqtSignal()
	mute = pyqtSignal()
	hide = pyqtSignal()
	delete = pyqtSignal()

class post_controls(QtWidgets.QWidget):
	def __init__(self, own_post=False, is_subscribed=False, parent=None):
		QtWidgets.QWidget.__init__(self, parent=parent)
		self.ui = Ui_post_controls()
		self.ui.setupUi(self)
		
		self.signals = signals()
		
		self.is_subscribed = is_subscribed
		self.subscriped_icon = QtGui.QIcon(QtGui.QPixmap(":/icons/icons/notify_post_active.svg"))
		self.unsubscriped_icon = QtGui.QIcon(QtGui.QPixmap(":/icons/icons/notify_post_normal.svg"))
		
		if own_post:
			self.ui.report.hide()										# hide report button
			self.ui.subscribe.hide()									# hide subscribe button
			self.ui.mute.hide()											# hide mute button
			self.ui.delete_.clicked.connect( self.delete )
		else:
			if self.is_subscribed:
				self.ui.subscribe.setIcon( self.subscriped_icon )
			else:
				self.ui.subscribe.setIcon( self.unsubscriped_icon )
			
			self.ui.delete_.clicked.connect( self.hide_post )
			self.ui.delete_.setToolTip("Click here to hide this post.")
			
			self.ui.report.clicked.connect( self.report )
			self.ui.report.setToolTip("Click here to report this post.")
			
			self.ui.subscribe.clicked.connect( self.subscribe )			# toggle
			
			self.ui.mute.clicked.connect( self.mute )
			self.ui.mute.setToolTip("Click here to block author of this post.")

	def confirm(self, title, quesion):
		answer = QtWidgets.QMessageBox.question(self, title, quesion, QtWidgets.QMessageBox.Yes, QtWidgets.QMessageBox.No)
		if answer == QtWidgets.QMessageBox.Yes:
			return True
		return False

	def hide_post(self):												# used if not own_post
		if self.confirm("Question", "Are you sure you want to hide this post?"):
			self.ui.delete_.clicked.disconnect()
			self.signals.hide.emit()

	def delete(self):													# used if own_post
		if self.confirm("Question", "Are you sure you want to delete this post?"):
			self.ui.delete_.clicked.disconnect()
			self.signals.delete.emit()

	def mute(self):
		if self.confirm("Question", "Are you sure you want block this author?"):
			self.ui.mute.clicked.disconnect()
			self.signals.mute.emit()

	def subscribe(self):
		if self.is_subscribed:
			self.signals.subscribe.emit(False)
		else:
			self.signals.subscribe.emit(True)

	def report(self):
		self.ui.report.clicked.disconnect()

	""" These functions will be called after remote action succesfull"""
	def unsubscribed(self):
		self.is_subscribed = False
		self.ui.subscribe.setIcon( self.unsubscriped_icon )
		self.ui.subscribe.setToolTip("Click to subscript to this post.")

	def subscribed(self):
		self.is_subscribed = True
		self.ui.subscribe.setIcon( self.subscriped_icon )
		self.ui.subscribe.setToolTip("Click to unsubscript to this post.")

	def reported(self):
		self.ui.report.hide()
		#TODO turn icon red.

	""" guess these functions can be deleted since to post will be
	deleted from stream """
	def muted(self):
		pass

	def deleted(self):
		pass

	def hided(self):
		pass
