# -*- coding: utf-8 -*-
"""
This file is part of LionsTooth.

LionsTooth is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

LionsTooth is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LionsTooth.  If not, see <http://www.gnu.org/licenses/>.
"""
from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtCore import pyqtSignal, QObject
from design.ui_post_interaction_bar import Ui_post_interaction_bar

class signals(QObject):
	like = pyqtSignal(bool)	# bool: like:unlike
	reshare = pyqtSignal()

class post_interaction_bar(QtWidgets.QWidget):
	def __init__(self, data, own_guid, from_reshare=False, own_post=False, public=False, parent=None):
		QtWidgets.QWidget.__init__(self, parent=parent)
		self.ui = Ui_post_interaction_bar()
		self.ui.setupUi(self)
		
		self.data = None
		self.public = public
		self.own_guid = own_guid
		self.own_post = own_post
		self.signals = signals()
		
		self.set_data(data)

		""" Hooks """ 
		self.ui.btn_like.clicked.connect( self.toggle_like )
		if not own_post and public: self.ui.btn_reshare.clicked.connect( self.reshare )
	
	def set_data(self, data):
		self.data = data

		""" Like """
		self.unliked()# reset
		if self.data['likes']:
			for like in self.data['likes']:
				# .. maybe add some small avatars
				if like['author']['guid'] == self.own_guid:
					self.ui.btn_like.setChecked(True)
					self.liked()
		self.ui.lbl_likes_count.setText( str(self.data['likes_count']) )

		""" Reshare 
		 - cannot reshare own posts """
		self.unreshared() # reset
		if self.own_post or not self.public:
			self.ui.btn_reshare.hide()
		elif self.data['reshares']:
			for reshare in self.data['reshares']:
				# .. maybe add some small avatars
				if reshare['author']['guid'] == self.own_guid:
					self.reshared()
		self.ui.lbl_shares_count.setText( str(self.data['reshares_count']) )

		""" Comments """
		self.ui.lbl_comments_count.setText( str(self.data['comments_count']) )


	""" Comment function """
	def commented(self):
		self.ui.lbl_comments_count.setText( str(int(self.ui.lbl_comments_count.text())+1) )
	
	def comment_deleted(self):
		self.ui.lbl_comments_count.setText( str(int(self.ui.lbl_comments_count.text())-1) )

	""" Reshare functions """
	def reshare(self):
		self.signals.reshare.emit()
		self.ui.btn_reshare.setEnabled(False)
	
	def reshared(self):
		self.ui.btn_reshare.hide()
		self.ui.lbl_shares_icon.setStyleSheet("color: #FF0000;")
		self.ui.btn_reshare.setEnabled(True)
	
	def unreshared(self):
		self.ui.btn_reshare.show()
		self.ui.lbl_shares_icon.setStyleSheet("color: #FFF;")
		self.ui.btn_reshare.setEnabled(True)
		
	def not_reshared(self):
		# TODO print failed message
		self.ui.btn_reshare.setEnabled(True)
	
	""" Like functions """
	def like(self):
		self.signals.like.emit(True)
		self.ui.btn_like.setEnabled(False)
		
	def unlike(self):
		self.signals.like.emit(False)
		self.ui.btn_like.setEnabled(False)
	
	def liked(self):
		self.ui.btn_like.setText("Liked")
		self.ui.lbl_likes_icon.setStyleSheet("color: #FF0000;")
		self.ui.lbl_likes_count.setText( str(int(self.ui.lbl_likes_count.text())+1) )
		self.ui.btn_like.setEnabled(True)
	
	def unliked(self):
		self.ui.btn_like.setText("Like")
		self.ui.lbl_likes_icon.setStyleSheet("color: #FFFFFF;")
		self.ui.lbl_likes_count.setText( str(int(self.ui.lbl_likes_count.text())-1) )
		self.ui.btn_like.setEnabled(True)
	
	def not_liked(self):
		# TODO print failed message
		self.ui.btn_like.setEnabled(True)
	
	def toggle_like(self, bool):
		if bool:
			self.like()
		else:
			self.unlike()
			
		
