"""
This file is part of LionsTooth.

LionsTooth is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

LionsTooth is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LionsTooth.  If not, see <http://www.gnu.org/licenses/>.
"""
from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtCore import pyqtSignal

class author_name(QtWidgets.QLineEdit): # this should be in common_widgets.py or something.
	clicked = pyqtSignal(str, str) # guid, diaspora_id
	def __init__(self, guid=None, diaspora_id=None, text=None, parent=None):
		self.guid = guid
		self.diaspora_id = diaspora_id
		QtWidgets.QLineEdit.__init__(self, parent)
		
		sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
		self.setSizePolicy(sizePolicy)
		
		self.font = QtGui.QFont("FreeSans", 10, QtGui.QFont.Bold)
		self.setFont(self.font)
		
		self.setReadOnly(True)
		self.setCursorMoveStyle(0) # set normal cursor http://pyqt.sourceforge.net/Docs/PyQt4/qt.html#CursorMoveStyle-enum
		self.setCursorPosition(0)
		self.setFrame(False)
		
		if diaspora_id:
			self.setToolTip(diaspora_id)
		if text:
			self.set_text(text)
		
	def mousePressEvent(self, event):
		self.clicked.emit(self.guid, self.diaspora_id)
	
	def set_text(self, text):
		self.setText(text)
		self.setCursorPosition(0)
	
	def set_guid(self, guid):
		self.guid = guid
