"""
This file is part of LionsTooth.

LionsTooth is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

LionsTooth is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LionsTooth.  If not, see <http://www.gnu.org/licenses/>.
"""
from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtCore import pyqtSignal

""" For getting image info like dimensions and frames (if animation) """
from PIL import Image

import re

class html_content(QtWidgets.QTextBrowser):
	updated_content = pyqtSignal()
	anchor_clicked = pyqtSignal(str)# str: anchor (the link)
	def __init__(self, content=None, parent=None):
		QtWidgets.QTextEdit.__init__(self, parent)
		self.setObjectName("html_content")

		""" document """
		self.document = QtGui.QTextDocument()
		self.document.setDefaultStyleSheet("a { color: #8ab854; }");
		self.document.setDocumentMargin(0)

		""" self """
		sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
		sizePolicy.setHorizontalStretch(0)
		sizePolicy.setVerticalStretch(0)
		sizePolicy.setHeightForWidth(self.sizePolicy().hasHeightForWidth())
		self.setSizePolicy(sizePolicy)

		self.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByKeyboard|QtCore.Qt.LinksAccessibleByMouse);
		self.setFrameShape(QtWidgets.QFrame.NoFrame)

		self.setReadOnly(True)
		self.setWordWrapMode(QtGui.QTextOption.WrapAtWordBoundaryOrAnywhere)

		self.setDocument(self.document)

		self.viewport().setMouseTracking(True)
		self.viewport().setCursor( QtCore.Qt.ArrowCursor )

		self.setOpenLinks(False) # If this is True and QTextBrowser can't open the link, it will remove the text.
		self.anchorClicked.connect( self._anchor_clicked )

	def _anchor_clicked(self, qurl): self.anchor_clicked.emit( qurl.url() )

	def update_html_content(self, html, emit=True):
		self.document.setHtml(html)
		if emit == True:
			self.updated_content.emit()

	def adjust_height(self):
		self.setMinimumHeight( self.document.size().height() )

	def resize_content(self, max_width): # only if added to layout
		self.document.setTextWidth( max_width )
		height = self.document.size().height()

		self.setMaximumWidth( max_width )

		self.setMinimumHeight( height )
		self.setMaximumHeight( height )

	def set_max_width(self, max_width):
		self.document.setTextWidth( max_width )
