"""
This file is part of LionsTooth.

LionsTooth is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

LionsTooth is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LionsTooth.  If not, see <http://www.gnu.org/licenses/>.
"""
from PyQt5 import QtCore, QtGui, QtWidgets

class wrappable_text(QtWidgets.QTextEdit):
	def __init__(self, parent=None):
		QtWidgets.QTextEdit.__init__(self, parent=parent)
		
		""" document """
		self.doc = QtGui.QTextDocument(self)
		self.doc.setDocumentMargin(0)
		
		""" self """
		sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
		sizePolicy.setHorizontalStretch(0)
		sizePolicy.setVerticalStretch(0)
		sizePolicy.setHeightForWidth(self.sizePolicy().hasHeightForWidth())
		self.setSizePolicy(sizePolicy)
		
		self.setFrameShape(QtWidgets.QFrame.NoFrame)
		
		self.setReadOnly(True)
		self.setLineWrapMode(True)
		self.setWordWrapMode(QtGui.QTextOption.WrapAtWordBoundaryOrAnywhere)

		self.setDocument(self.doc)
		
		self.setMinimumHeight( 20 )
	
	def set_bold(self, bool):
		default_font = self.doc.defaultFont()
		if bool:
			self.doc.setDefaultFont( QtGui.QFont(default_font.family(), default_font.pointSize(), QtGui.QFont.Bold) )
		else:
			self.doc.setDefaultFont( QtGui.QFont(default_font.family(), default_font.pointSize()) )
		
	def update_text(self, text):
		self.doc.setPlainText( text )

	def fit_height(self):
		self.adjustSize()
		
		parent_widget = self.parentWidget()
		width = parent_widget.size().width()
		
		parent_widget_layout_margins = parent_widget.layout().contentsMargins()
		width -= parent_widget_layout_margins.left()
		width -= parent_widget_layout_margins.right()
		
		parent_widget_margins = parent_widget.contentsMargins()
		width -= parent_widget_margins.left()
		width -= parent_widget_margins.right()

		self.doc.setTextWidth( width )
		
		doc_size = self.doc.size()
		doc_height = doc_size.height()

		self.setMinimumHeight( doc_height )
		self.setMaximumHeight( doc_height )
